////////////////////////////////////////////////////////////////////////////////
/// @brief class for result blobs
///
/// @file
///
/// DISCLAIMER
///
/// Copyright 2004-2012 triagens GmbH, Cologne, Germany
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///
///     http://www.apache.org/licenses/LICENSE-2.0
///
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// Copyright holder is triAGENS GmbH, Cologne, Germany
///
/// @author Dr. Frank Celler
/// @author Achim Brandt
/// @author Copyright 2008-2012, triAGENS GmbH, Cologne, Germany
////////////////////////////////////////////////////////////////////////////////

#include "VariantBlob.h"

#include "Basics/StringBuffer.h"

using namespace std;
using namespace triagens::basics;

// -----------------------------------------------------------------------------
// --SECTION--                                      constructors and destructors
// -----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
/// @addtogroup Variants
/// @{
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
/// @brief constructs a new blob
////////////////////////////////////////////////////////////////////////////////

VariantBlob::VariantBlob ()
  : _value(0), _length(0) {
}

////////////////////////////////////////////////////////////////////////////////
/// @brief constructs a new blob
////////////////////////////////////////////////////////////////////////////////

VariantBlob::VariantBlob (char const* pointer, size_t len)
  : _value(0), _length(0) {
  if (pointer != 0) {
    _value = new char[len];
    memcpy(_value, pointer, len);
    _length = len;
  }
}

////////////////////////////////////////////////////////////////////////////////
/// @brief destructs a blob
////////////////////////////////////////////////////////////////////////////////

VariantBlob::~VariantBlob() {
  if (_value) {
    delete[] _value;
  }
}

////////////////////////////////////////////////////////////////////////////////
/// @}
////////////////////////////////////////////////////////////////////////////////

// -----------------------------------------------------------------------------
// --SECTION--                                             VariantObject methods
// -----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
/// @addtogroup Variants
/// @{
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
/// {@inheritDoc}
////////////////////////////////////////////////////////////////////////////////

VariantObject* VariantBlob::clone () const {
  return new VariantBlob(_value, _length);
}

////////////////////////////////////////////////////////////////////////////////
/// {@inheritDoc}
////////////////////////////////////////////////////////////////////////////////

void VariantBlob::print (StringBuffer& buffer, size_t) const {
  buffer.appendText("(blob) ");
  buffer.appendInteger(_length);
  buffer.appendEol();
}

////////////////////////////////////////////////////////////////////////////////
/// @}
////////////////////////////////////////////////////////////////////////////////

// -----------------------------------------------------------------------------
// --SECTION--                                                    public methods
// -----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
/// @addtogroup Variants
/// @{
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
/// @brief returns the _value
////////////////////////////////////////////////////////////////////////////////

const char* VariantBlob::getValue () const {
  return _value;
}

////////////////////////////////////////////////////////////////////////////////
/// @brief returns the _length
////////////////////////////////////////////////////////////////////////////////

size_t VariantBlob::getLength () const {
  return _length;
}

////////////////////////////////////////////////////////////////////////////////
/// @}
////////////////////////////////////////////////////////////////////////////////

// Local Variables:
// mode: outline-minor
// outline-regexp: "^\\(/// @brief\\|/// {@inheritDoc}\\|/// @addtogroup\\|// --SECTION--\\|/// @\\}\\)"
// End:
